"""Command for contacts importation"""

import os
import codecs

from django.core.management.base import BaseCommand
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as ugt
from django.contrib.admin import ModelAdmin
from django.contrib.admin import site
from django.contrib.admin.util import lookup_field, display_for_field
from django.utils.html import escape, strip_tags
from django.utils.encoding import force_unicode, smart_unicode
from django.core.mail import send_mail
from django.core.mail.message import EmailMessage

from optparse import make_option

from acomea.contact.utils import importation
from acomea.contact.admin import Contact, ContactAdmin

def export_csv(modeladmin, output_file, limit=0, verbosity=0):

    queryset = Contact.objects.all().select_related('network', 'category', 'email',  'phone', 'phone__category')
    
    #prefetch_related('groups', 'interests', 'products',)
    if limit:
        queryset = queryset[0:limit]

    if hasattr(modeladmin, 'list_display_csv'):
        fields_list = modeladmin.list_display_csv
    else:
        fields_list = modeladmin.list_display
    plural_name = ugt(modeladmin.model._meta.verbose_name_plural)
    rows = []

    title = list(fields_list)
    title.append('\n')
    if 'action_checkbox' in title:
        title.remove('action_checkbox')
    title = ";".join(title)
    rows.append(title)

    for obj in queryset:
        row = []
        for field in fields_list:
            if field == 'action_checkbox':
                continue
            f, attr, value = lookup_field(field, obj, modeladmin)
            if f is None:
                result_repr = strip_tags(smart_unicode(value))
            else:
                result_repr = display_for_field(value, f) if value else ''

            # Tolgo eventuali 'a capo'
            result_repr = result_repr.replace('\r\n', ' ')
            row.append(result_repr)
        row = ";".join(row)
        rows.append(row)

    f = codecs.open(output_file, 'w', 'utf-8')
    f.write('\n'.join(rows))
    f.close()
    if verbosity >= 1:
        print "Wrote file %s" % (os.path.abspath(output_file,))

class Command(BaseCommand):
    """..."""
    help = 'Export contacts from file. Fixed structure'
    option_list = BaseCommand.option_list + (
        make_option(
            '-f', '--file', dest='output_file',
            help=_('file to create')
        ),
        make_option(
            '-e', '--email', dest='email',
            help=_('Email where output should be sent')
        ),
        make_option('-l', '--limit', dest='limit', help='Limit contacts', default=0),
    )

    def handle(self, *scripts, **opts):
        output_file = opts.get('output_file')
        limit = opts.get('limit')
        
        export_csv(ContactAdmin(Contact, site), output_file, limit=limit, verbosity=opts.get('verbosity'))

        if opts.get('email'):
            msg = EmailMessage('Export contatti', '', '', [opts.get('email')])
            msg.attach_file(output_file)
            msg.send(fail_silently=False)
