from django.db import models
from django.utils.html import escape, strip_tags
from django.utils.encoding import force_unicode, smart_unicode
from django.utils.translation import ugettext as ugt
from django.contrib.admin.util import unquote, lookup_field, display_for_field, get_deleted_objects
from jmb.core.utils.functions import render_csv, render_xls

def _export_data(modeladmin, request, queryset):
    """
    Ritorna una lista di liste contenente i valori degli item del queryset
    """
    if hasattr(modeladmin, 'list_display_csv'):
        fields_list = modeladmin.list_display_csv
    else:
        fields_list = modeladmin.list_display

    plural_name = ugt(modeladmin.model._meta.verbose_name_plural)

    title = list(fields_list)
    title.append('\n')
    if 'action_checkbox' in title:
        title.remove('action_checkbox')
    yield title

    for obj in queryset:
        row = []
        for field in fields_list:
                
            if field == 'action_checkbox':
                continue
            f, attr, value = lookup_field(field, obj, modeladmin)
            if not f:
                # try to interpret it as a modeladmin's function
                if hasattr(modeladmin, field):
                    result_repr = getattr(modeladmin, field)(obj)
            elif value is None:
                if isinstance(f, (models.DecimalField, models.IntegerField, models.FloatField, models.NullBooleanField)):
                    result_repr = ''
                elif f is None:
                    result_repr = strip_tags(smart_unicode(value))
            else:
                if isinstance(f, (models.NullBooleanField, models.BooleanField)):
                    result_repr = False and _('No') or _('Yes')
                else:
                    result_repr = display_for_field(value, f)

            # Tolgo eventuali 'a capo'
            row.append(unicode(result_repr))
        yield row



