# -*- coding: utf-8 -*-
"""
contacts generic utils

"""
import logging
import base64

from django.conf import settings
from django.utils.translation import ugettext_lazy as _
# from django.contrib.auth.tokens import default_token_generator
from django.utils.http import urlencode
from django.template.loader import get_template, render_to_string
from django.template import Context
from django.core.mail import EmailMultiAlternatives
from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned
from django.template.loader import get_template, render_to_string
from django.contrib.sites.models import get_current_site
from django.utils.http import int_to_base36, base36_to_int
from django.contrib.auth.models import User

from web.utils.generic import get_acomea_host, get_acomea_tcp_protocol

from acomea.contact.models import Contact, Email
from acomea.contact.utils.tokens import contact_token_generator

#######################
# GENERIC UTILS       #
#######################

log = logging.getLogger("acomea.contact")

GRUPPO_CLI_CONS_NAME = "CLIENTI_CONS"
GRUPPO_CLI_DISP_NAME = "CLIENTI_DISP"
GRUPPO_LEAD_NAME = "Lead"
GRUPPO_AR_NAME = "Utenti Registrati AR"

def send_contact_confirm_email(
        contact, email, request,
        confirm_page_url=None,
        template=None,
        subject=None,
        **kwargs
):
    """
    send email to contact for verification
    """
    alternate_url = kwargs.pop('alternate_url', None)
    # new_contact = kwargs.pop('new_contact', None)
    internal_file = kwargs.pop('internal_file', None)
    values_string = kwargs.pop('values_string', None)
    # confirmed_contact = not new_contact and contact.bonifica
    if not template:
        template = settings.CONTACT_CONFIRM_EMAIL_TEMPLATE
    try:
        html_template = get_template(template)
        text_template = template.split('.')[0] + '.txt'
    except Exception, e:
        log.error(
            "CONTACT_EMAIL_CONFIRM_TEMPLATE missing or wrong in settings. msg: %s" % e.message
        )
        return
    if not confirm_page_url:
        confirm_page_url = settings.CONTACT_CONFIRM_LANDINGPAGE_RELATIVE_URL

    current_site = get_current_site(request)
    site_name = current_site.name
    page_relative_url = request.get_full_path()
    cidb36 = int_to_base36(contact.id)
    token = contact_token_generator.make_token(contact)
    protocol = get_acomea_tcp_protocol(request)
    domain = current_site.domain

    if internal_file:
        alternate_url = "%s://%s%s" % (protocol, get_acomea_host(), alternate_url)

    confirm_url = "%(protocol)s://%(acomea_host)s%(confirm_page_url)s?cidb36=%(cidb36)s&token=%(token)s" % {
        'protocol': protocol, 'acomea_host': get_acomea_host(),
        'confirm_page_url': str(confirm_page_url),
        'cidb36': str(cidb36), 'token': str(token)
    }

    if alternate_url:
        alternate_url = base64.b64encode(alternate_url)
        confirm_url += "&alternate_url=%s&encoded=1" % alternate_url

    if values_string:
        confirm_url += "&values=%s" % values_string

    ctx_dict = Context({
        'email': email,
        'domain': domain,
        'site_name': site_name,
        'contact': contact,
        'token': token,
        'protocol': protocol,
        'confirm_url': confirm_url,
        'static': "%s://%s%s" % (protocol, get_acomea_host(), settings.STATIC_URL),
        'homepage': "%s://%s" % (protocol, get_acomea_host()),
        'confirmed_contact': contact.bonifica,
        'alternate_url': alternate_url
    })

    if not subject:
        subject = "Iscrizione newsletter %s " % ctx_dict['site_name']
    try:
        from_email = settings.CONTACT_CONFIRM_EMAIL_SENT_FROM
    except AttributeError,e:
        from_email = 'info@acomea.it'
    to = [email]
    try:
        # debug setting: to define in local settings: send email to this address
        # instead of real address
        to = [settings.CONTACT_CONFIRM_EMAIL_SENT_TO]
    except AttributeError:
        pass

    text_message = render_to_string(text_template, ctx_dict)
    html_message = html_template.render(ctx_dict)
    msg = EmailMultiAlternatives(subject, text_message, from_email, to)
    msg.attach_alternative(html_message, "text/html")
    msg.send()

def check_contact_confirm_url(request, method=None):
    """
    check confirm url for a contact
    """
    query_dict = None
    if not method:
        method = 'GET'
    query_dict = getattr(request, method, None)


    if  query_dict and (
        'cidb36' in query_dict and
        'token' in query_dict
        ):
        try:
            cid_int = base36_to_int(query_dict['cidb36'])
            _contact = Contact.objects.get(id=cid_int)
            if contact_token_generator.check_token(_contact, query_dict['token']):
                return _contact
            else:
                return False
        except (ValueError, Contact.DoesNotExist) as exc:
            # il contatto non esiste / errore
            return False

def send_confirm_privacydata_contact(contact, email, request, send_to=None, data_confirmed=None):
    """
    bonifica e-mail (confirm privacy data email)
    """
    try:
        html_template = get_template(settings.CONTACT_CMSAPPS_CONFIRMPRIVACY_EMAIL_TEMPLATE)
        text_template = settings.CONTACT_CMSAPPS_CONFIRMPRIVACY_EMAIL_TEMPLATE.split('.')[0] + '.txt'
    except Exception, e:
        log.error(
            "CONTACT_CMSAPPS_CONFIRMPRIVACY_EMAIL_TEMPLATE missing or wrong in settings. msg: %s" % e.message
        )
        return None
    current_site = get_current_site(request)
    site_name = current_site.name
    cidb36 = int_to_base36(contact.id)
    token = contact_token_generator.make_token(contact)
    protocol = get_acomea_tcp_protocol(request)

    # FIXME: remove hardcoded urls
    confirm_url = "/registrazione/conferma-privacy"
    confirm_url = "%s://%s%s?email=%s&cidb36=%s&token=%s" % (
        protocol,
        get_acomea_host(),
        confirm_url,
        email,
        str(cidb36),
        token
    )

    ctx_dict = Context({
        'email': email,
        'site_name': site_name,
        'contact': contact,
        'protocol': protocol,
        'confirm_url': confirm_url,
        'data_confirmed':data_confirmed,
        'static': protocol + "://" + get_acomea_host() + settings.STATIC_URL,
        'homepage': protocol + "://" + get_acomea_host()
    })
    if data_confirmed:
        subject = "Dati contatto confermati - AcomeA"
    else:
        subject = "Rimani in contatto con AcomeA"
    try:
        from_email = settings.CONTACT_CONFIRM_EMAIL_SENT_FROM
    except AttributeError,e:
        from_email = 'privacy@acomea.it'

    try:
        # debug setting: to define in local settings: send email to this address
        # instead of real address
        to = [settings.CONTACT_CMSAPPS_CONFIRMPRIVACY_SENT_TO]
    except AttributeError:
        to = [email]

    text_message = render_to_string(text_template, ctx_dict)
    html_message = html_template.render(ctx_dict)
    msg = EmailMultiAlternatives(subject, text_message, from_email, to)
    msg.attach_alternative(html_message, "text/html")
    msg.send()

def get_new_token_and_cidb36(contact):
    """
    """
    token = contact_token_generator.make_token(contact)
    cidb36 = int_to_base36(contact.id)
    return (token, cidb36)



def send_delete_contact_notify_to_backoffice(contact):
    """
    send notify email to backoffice for contact deleting

    """
    try:
        html_template = get_template(settings.CONTACT_CMSAPPS_CONFIRMPRIVACY_CONTACT_DELETE_TO_BACKOFFICE_EMAIL_TEMPLATE)
        text_template = settings.CONTACT_CMSAPPS_CONFIRMPRIVACY_CONTACT_DELETE_TO_BACKOFFICE_EMAIL_TEMPLATE.split('.')[0] + '.txt'
    except Exception, e:
        log.error(
            "CONTACT_CMSAPPS_CONFIRMPRIVACY_CONTACT_DELETE_TO_BACKOFFICE_EMAIL_TEMPLATE missing or wrong in settings. msg: %s" % e.message
        )
        return None

    ctx_dict = Context({
        'contact': contact,
    })

    subject = "Eliminazione contatto (tramite Conferma Privacy) - %s %s " % (contact.first_name, contact.last_name)
    try:
        from_email = settings.CONTACT_CONFIRM_EMAIL_SENT_FROM
    except AttributeError,e:
        from_email = 'info@acomea.it'

    try:
        # debug setting: to define in local settings: send email to this address
        # instead of real address
        to = [settings.CONTACT_CMSAPPS_CONFIRMPRIVACY_CONTACT_DELETE_TO_BACKOFFICE_EMAIL_SENT_TO]
    except AttributeError:
        to = ['privacy@acomea.it']

    text_message = render_to_string(text_template, ctx_dict)
    html_message = html_template.render(ctx_dict)
    msg = EmailMultiAlternatives(subject, text_message, from_email, to)
    msg.attach_alternative(html_message, "text/html")
    msg.send()



def has_related_user(contact):

    try:
        user = contact.user
        if not user:
            return False
    except User.DoesNotExist:
        return False
    return True



## FIXME: codice preso da acomea.profilo.utils -> da spostare in helper comune perche' riutilizzato in piu' parti
#def get_acomea_host():
#    try:
#        return settings.PROFILO_HOST
#    except AttributeError:
#        pass
#    return "127.0.0.1:8000"
#
#
## FIXME: codice preso da acomea.profilo.utils -> da spostare in helper comune perche' riutilizzato in piu' parti
#def get_acomea_tcp_protocol(request):
#    try:
#        return settings.PROFILO_TCP_PROTOCOL
#    except AttributeError:
#        pass
#    return "https" if request.is_secure() else "http"


def bonifica_gruppi_contact_lead_disp_cons():
    """
    bonifica i contatti associati agli user in base a stato rac o se sono solo lead
    - se solo lead: aggiungi contatto al gruppo 'Lead'
    - se cl. consultivo (rac,stato=6): aggiungi contatto al gruppo 'CLIENTI_CONS'
    - se cl. dispositivo (rac,stato=9): aggiungi contatto al gruppo 'CLIENTI_DISP'
    """
    from acomea.profilo.models import RichiestaAttivazioneCliente, AcomeaProfilo
    from django.contrib.auth.models import User

    success = 0
    to_disp = 0
    to_cons = 0
    to_lead = 0
    errors = 0

    print "***********  bonifica gruppi contact - lead/dispositivi/consultivi  ***********"
    for rac in RichiestaAttivazioneCliente.objects.all():
        try:
            profilo = rac.profilo
            user = profilo.user
            if profilo.is_only_lead:
                profilo.aggiungi_a_gruppo_contatti(group_name=GRUPPO_LEAD_NAME, email=user.email)
                success += 1
                to_lead += 1
                continue
            if rac.stato == "6":
                profilo.aggiungi_a_gruppo_contatti(group_name=GRUPPO_CLI_CONS_NAME, email=user.email)
                success += 1
                to_cons += 1
                continue
            if rac.stato == "9":
                profilo.aggiungi_a_gruppo_contatti(group_name=GRUPPO_CLI_DISP_NAME, email=user.email)
                success += 1
                to_disp += 1
                continue
        except AcomeaProfilo.DoesNotExist:
            print 'ERR: la rac %s non ha associato alcun profilo' % str(rac.id)
            errors +=1
            continue
        except User.DoesNotExist:
            print 'ERR: il profilo %s non ha associato alcuno user' % str(profilo.id)
            errors +=1
            continue
        except Exception, e:
            print "ERR: errore -> %s" % e.message
            errors += 1
            continue

    print "bonifica effettuata"
    print "aggiunte a gruppo %s: %s " % (GRUPPO_LEAD_NAME, str(to_lead))
    print "aggiunte a gruppo %s: %s " % (GRUPPO_CLI_CONS_NAME, str(to_cons))
    print "aggiunte a gruppo %s: %s " % (GRUPPO_CLI_DISP_NAME, str(to_disp))
    print "totali: %s " % str(success)
    print "non effettuate / errori: %s " % str(errors)

                # self.profilo.rimuovi_da_gruppo_contatti(group_name="Lead")
                # self.profilo.rimuovi_da_gruppo_contatti(group_name="CLIENTI_CONS")
                # self.profilo.aggiungi_a_gruppo_contatti(group_name="CLIENTI_DISP", email=self.profilo.user.email)

def bonifica_contatti_non_utenti_ar():
    """
    se contact ha associato uno user e lo user è attivo, associa a gruppo
    """
    GRUPPO_AR_NAME = "Utenti Registrati AR"
    from acomea.contact.models import Contact, Group
    from django.contrib.auth.models import User
    bonificati = 0
    non_bonificati = 0
    error = 0
    try:
        group = Group.objects.get(name__iexact=GRUPPO_AR_NAME)
    except Group.DoesNotExist:
        print "Gruppo con nome %s non trovato" % GRUPPO_AR_NAME
        return False
    contact_qs = Contact.objects.filter(user_id__isnull=False).exclude(groups__name__in=[GRUPPO_AR_NAME])
    for contact in contact_qs:
        try:
            user = contact.user
            if user.id and user.is_active:
                contact.groups.add(group)
                bonificati += 1
            else:
                non_bonificati += 1
        except User.DoesNotExist:
            print "Contatto %s non associato a user" % contact
            error += 1

    print "bonificati %s contatti" % str(bonificati)
    print "non bonificati: %s" % str(non_bonificati)
    print "errori: %s" % str(error)



def bonifica_contatti_senza_user():
    """
    cicla i contatti senza user associato, per ognuno di essi cicla le mail associate ad esso:
    se esiste uno user con quella mail:
    - associa lo user al contatto
    - setta la mail come default
    - setta i flag bonifica, auth_trattamento_dati, auth_materiale_commerciale a True
    - aggiunge il contatto al gruppo utenti registrati
    - aggiunge il contatto ai gruppi clienti dispositivi/consultivi in base allo stato della rac
    """

    from django.db import connection, IntegrityError, DatabaseError
    from django.contrib.auth.models import User

    from acomea.contact.models import Contact, Group, Email
    from acomea.profilo.models import AcomeaProfilo, RichiestaAttivazioneCliente
    print "********** bonifica_contatti_senza_user **********"
    print ""
    # gruppi
    gruppo_ar = gruppo_lead = gruppo_cons = gruppo_disp = None
    group_name_list = [GRUPPO_AR_NAME, GRUPPO_LEAD_NAME, GRUPPO_CLI_DISP_NAME, GRUPPO_CLI_CONS_NAME]
    group_list = [gruppo_ar, gruppo_lead, gruppo_disp, gruppo_cons]
    for group, name in zip(group_list, group_name_list):
        try:
            group = Group.objects.get(name=name)
        except Group.DoesNotExist:
            print "gruppo con nome %s non trovato" % name
            continue
        except Group.MultipleObjectsReturned:
            print "piu' gruppi con nome %s trovati" % name
            continue

    bonificati = 0
    non_bonificati = 0
    errori = 0
    for contact in Contact.objects.filter(user__isnull=True):
        try:
            for email_obj in contact.emails.all():
                user = User.objects.get(email=email_obj.email)
                contact.user = user
                email_obj.default = True
                contact.bonifica = True
                contact.auth_trattamento_dati = True
                contact.auth_materiale_commerciale = True

                if gruppo_ar:
                    contact.groups.add(gruppo_ar)
                if user.profilo.is_only_lead:
                    if gruppo_lead:
                        contact.groups.add(gruppo_lead)
                if user.profilo.rac.stato == "9":
                    if gruppo_disp:
                        contact.groups.add(gruppo_disp)
                if user.profilo.rac.stato == "6":
                    if gruppo_cons:
                        contact.groups.add(gruppo_cons)

                email_obj.save()
                contact.save()
                bonificati += 1
        except User.DoesNotExist:
            pass
            # print "non esiste user con mail %s" % e.email
        except User.MultipleObjectsReturned:
            print "piu' user con mail %s" % email_obj.email
            errori += 1
        except Email.DoesNotExist:
            print "non ci sono email associate al contatto %s" % contact
            errori += 1
        except AcomeaProfilo.DoesNotExist:
            print "non ci sono profili associati allo user %s" % user
            errori += 1
        except RichiestaAttivazioneCliente.DoesNotExist:
            print "non ci sono rac associate al profilo %s" % user.profilo
            errori += 1
        except IntegrityError, e:
            print "ERR INT: %s -- %s" % (e.message, str(e.args))
            errori += 1
        except DatabaseError, e:
            print "ERR DAT: %s -- %s" % (e.message, str(e.args))
            connection._rollback()
            errori += 1

    print ""
    print "* bonifica effettuata *"
    print "bonificati: %d" % bonificati
    print "errori: %d" % errori


def set_groups_for_contact(contact, user):
    """
    setta gruppi per contatto
    """
    from django.db import IntegrityError, DatabaseError
    from django.db import connection

    from django.contrib.auth.models import User
    from acomea.contact.models import Contact, Group, Email
    from acomea.profilo.models import AcomeaProfilo, RichiestaAttivazioneCliente

    # gruppo utenti registrati
    try:
        gruppo_ar = Group.objects.get(name=GRUPPO_AR_NAME)
        contact.groups.add(gruppo_ar)
    except Group.DoesNotExist:
        log.error("gruppo con nome %s non trovato" % GRUPPO_AR_NAME)
    except Group.MultipleObjectsReturned:
        log.error("piu' gruppi con nome %s trovati" % GRUPPO_AR_NAME)

    # gruppo lead
    try:
        if user.profilo.is_only_lead:
            gruppo_lead = Group.objects.get(name=GRUPPO_LEAD_NAME)
            contact.groups.add(gruppo_lead)
    except User.DoesNotExist:
        pass
    except AcomeaProfilo.DoesNotExist:
        pass
    except Group.DoesNotExist:
        log.error("gruppo con nome %s non trovato" % GRUPPO_LEAD_NAME)
    except Group.MultipleObjectsReturned:
        log.error("piu' gruppi con nome %s trovati" % GRUPPO_LEAD_NAME)


    # gruppo clienti disp. / cons.
    try:
        if user.profilo.rac.stato == "9":
            gruppo_disp = Group.objects.get(name=GRUPPO_CLI_DISP_NAME)
            contact.groups.add(gruppo_disp)
        if user.profilo.rac.stato == "6":
            gruppo_cons = Group.objects.get(name=GRUPPO_CLI_CONS_NAME)
            contact.groups.add(gruppo_cons)
    except AcomeaProfilo.DoesNotExist:
        pass
    except RichiestaAttivazioneCliente.DoesNotExist:
        pass
    except Group.DoesNotExist:
        log.error("gruppo con nome %s o %s non trovato" % (GRUPPO_CLI_DISP_NAME, GRUPPO_CLI_CONS_NAME))
    except Group.MultipleObjectsReturned:
        log.error("piu' gruppi con nome %s o %s trovati" % (GRUPPO_CLI_DISP_NAME, GRUPPO_CLI_CONS_NAME))


def is_new_contact(email):
    """
    check if there's already and existant email with a contact associated
    """
    try:
        email = Email.objects.get(email=email.lower())
        if email.contact and email.contact.id:
            return False
        else:
            return True
    except(ObjectDoesNotExist, MultipleObjectsReturned):
        return True