#-*- coding: utf-8 -*-

from django.contrib import messages
from django.template import loader
from django.template.base import Context
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe

from jmb.mailer.sender import Mailer
from acomea.event.models import Invite


def get_queryset(obj, request, queryset):
    if request.POST.get('select_across') == '1':
        # se sono stati selezionati tutti i contatti (anche delle pagine successive)
        # quindi occorre ricostruire il queryset
        queryset = obj.queryset(request)
    return queryset


def render_template_from_string(context_dict, body):
    if 'invite' in context_dict and hasattr(context_dict['invite'], "get_landingpage"):
        context_dict['invite'].get_landingpage = mark_safe(context_dict['invite'].get_landingpage())
    context = Context(context_dict)
    template = loader.get_template_from_string(body)
    ret = template.render(context)
    return ret if not "<code></code>" in ret else ret.replace("<code></code>", "")


def print_messages(request, event, sent_in_test, create, not_create, test):
    # PRINT MESSAGES
    if create == 0 and not test:
        messages.error(request, _('%s not create') % event)
    if sent_in_test == 0 and test:
        messages.error(request, _('%s not sent in test') % event)
    if create > 0 and not test:
        messages.info(request, _('{0} succesfully create to {1} contacts').format(
            event, create)
        )
    if sent_in_test > 0 and test:
        messages.info(request, _('{0} succesfully sent in test to {1} contacts').format(
            event, sent_in_test)
        )
    if not_create > 0:
        messages.error(request, _('{0}: {1} in error').format(event, not_create))
    return


def send_invite_to_contact(contact, event, test, request=None):
    sent_in_test = False
    created = False
    not_created = False
    mailer = Mailer(event, test, is_task=True, request=request)
    invite, invite_created = Invite.objects.get_or_create(event=event, contact=contact)
    if invite_created:
        adhesion = event.translations.get(
            language__id=event.default_language.id,
        )
        description = render_template_from_string(
            dict(contact=contact, invite=invite, is_email=True),
            adhesion.body
        )
        context = dict(description=description, invite=invite)
        mail_created, email_object = mailer.create_message(contact, context)
        if mail_created:
            created = True
            # se test - invio il messaggio immediatamente
            if test:
                mailer.send(email_object)
                sent_in_test = True
                invite.sent = True
                invite.save()
        else:
            not_created = True
    return dict(sent_in_test=sent_in_test, created=created, not_created=not_created)