# -*- coding: utf-8 -*-
"""
contacts token generator utils

"""
from datetime import date
from django.conf import settings
from django.utils.http import int_to_base36, base36_to_int
from django.utils.crypto import constant_time_compare, salted_hmac
from django.contrib.auth.tokens import PasswordResetTokenGenerator

class ContactTokenGenerator(PasswordResetTokenGenerator):
    """

    """

    def make_token(self, contact):
        """
        overrides of original make_token method. Passing contact instance instead of user
        """
        return self._make_token_with_timestamp(contact, self._num_days(self._today()))

    def _make_token_with_timestamp(self, contact, timestamp):
        """
        overrides of original _make_token_with_timestamp. Passing contact instance instead of user,
        used a different key_salt, used last_modify_timestamp instead of login_timestamp
        """

        # timestamp is number of days since 2001-1-1.  Converted to
        # base 36, this gives us a 3 digit string until about 2121
        ts_b36 = int_to_base36(timestamp)

        key_salt = "acomea.contact.utils.tokens.ContactTokenGenerator"
        last_modify_timestamp = contact.date_last_modify.replace(microsecond=0, tzinfo=None)

        value = (unicode(contact.id) +
                 unicode(contact.bonifica) +
                 unicode(last_modify_timestamp) +
                 unicode(timestamp))
        hash = salted_hmac(key_salt, value).hexdigest()[::2]
        return "%s-%s" % (ts_b36, hash)

    def check_token(self, contact, token):
        """
        Check that a  token is correct for a given contact.
        """
        # Parse the token
        try:
            ts_b36, hash = token.split("-")
        except ValueError:
            return False

        try:
            ts = base36_to_int(ts_b36)
        except ValueError:
            return False

        # Check that the timestamp/cid has not been tampered with
        #ctcp = constant_time_compare(self._make_token_with_timestamp(contact, ts), token)
        if not constant_time_compare(self._make_token_with_timestamp(contact, ts), token):
            return False

        # Check the timestamp is within limit
        if (self._num_days(self._today()) - ts) > settings.CONTACT_CONFIRM_TIMEOUT_DAYS:
            return False

        return True


contact_token_generator = ContactTokenGenerator()